@@ -33,6 +33,7 @@
 #include <setjmp.h>
 #include <stdbool.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <stdarg.h>
@@ -44,6 +45,8 @@
 
 
 extern const char* __igt_test_description __attribute__((weak));
+extern bool __igt_plain_output;
+
 
 /**
  * IGT_TEST_DESCRIPTION:
@@ -83,6 +86,12 @@ extern const char* __igt_test_description __attribute__((weak));
  */
 #define IGT_EXIT_INVALID 79
 
+/**
+ * IGT_EXIT_FAILURE
+ *
+ * Exit status indicating a test failure
+ */
+#define IGT_EXIT_FAILURE 99
 
 bool __igt_fixture(void);
 void __igt_fixture_complete(void);
@@ -93,20 +102,20 @@ void __igt_fixture_end(void) __attribute__((noreturn));
  * Annotate global test fixture code
  *
  * Testcase with subtests often need to set up a bunch of global state as the
- * common test fixture. To avoid such code interferring with the subtest
- * enumeration (e.g. when enumerating on systemes without an intel gpu) such
+ * common test fixture. To avoid such code interfering with the subtest
+ * enumeration (e.g. when enumerating on systems without an intel gpu) such
  * blocks should be annotated with igt_fixture.
  */
 #define igt_fixture for (int igt_tokencat(__tmpint,__LINE__) = 0; \
 			 igt_tokencat(__tmpint,__LINE__) < 1 && \
 			 __igt_fixture() && \
-			 (setjmp(igt_subtest_jmpbuf) == 0); \
+			 (sigsetjmp(igt_subtest_jmpbuf, 1) == 0); \
 			 igt_tokencat(__tmpint,__LINE__) ++, \
 			 __igt_fixture_complete())
 
 /* subtest infrastructure */
 jmp_buf igt_subtest_jmpbuf;
-typedef int (*igt_opt_handler_t)(int opt, int opt_index);
+typedef int (*igt_opt_handler_t)(int opt, int opt_index, void *data);
 #ifndef __GTK_DOC_IGNORE__ /* gtkdoc wants to document this forward decl */
 struct option;
 #endif
@@ -114,7 +123,8 @@ int igt_subtest_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
 				struct option *extra_long_opts,
 				const char *help_str,
-				igt_opt_handler_t extra_opt_handler);
+				igt_opt_handler_t extra_opt_handler,
+				void *handler_data);
 
 
 /**
@@ -123,14 +133,15 @@ int igt_subtest_init_parse_opts(int *argc, char **argv,
  * @argv: argv from the test's main()
  *
  * This initializes the for tests with subtests without the need for additional
- * cmdline options. It is just a simplified version of
+ * command line options. It is just a simplified version of
  * igt_subtest_init_parse_opts().
  *
  * If there's not a reason to the contrary it's less error prone to just use an
  * #igt_main block instead of stitching the test's main() function together
  * manually.
  */
-#define igt_subtest_init(argc, argv) igt_subtest_init_parse_opts(&argc, argv, NULL, NULL, NULL, NULL);
+#define igt_subtest_init(argc, argv) \
+	igt_subtest_init_parse_opts(&argc, argv, NULL, NULL, NULL, NULL, NULL);
 
 bool __igt_run_subtest(const char *subtest_name);
 #define __igt_tokencat2(x, y) x ## y
@@ -157,14 +168,14 @@ bool __igt_run_subtest(const char *subtest_name);
  * This is a simpler version of igt_subtest_f()
  */
 #define igt_subtest(name) for (; __igt_run_subtest((name)) && \
-				   (setjmp(igt_subtest_jmpbuf) == 0); \
+				   (sigsetjmp(igt_subtest_jmpbuf, 1) == 0); \
 				   igt_success())
 #define __igt_subtest_f(tmp, format...) \
 	for (char tmp [256]; \
 	     snprintf( tmp , sizeof( tmp ), \
 		      format), \
 	     __igt_run_subtest( tmp ) && \
-	     (setjmp(igt_subtest_jmpbuf) == 0); \
+	     (sigsetjmp(igt_subtest_jmpbuf, 1) == 0); \
 	     igt_success())
 
 /**
@@ -190,23 +201,26 @@ bool igt_only_list_subtests(void);
  *
  * This is a magic control flow block used instead of a main() function for
  * tests with subtests. Open-coding the main() function is only recommended if
- * the test needs to parse additional cmdline arguments of its own.
+ * the test needs to parse additional command line arguments of its own.
  */
 #define igt_main \
 	static void igt_tokencat(__real_main, __LINE__)(void); \
 	int main(int argc, char **argv) { \
-		igt_subtest_init_parse_opts(&argc, argv, NULL, NULL, NULL, NULL); \
+		igt_subtest_init_parse_opts(&argc, argv, NULL, NULL, NULL, \
+					    NULL, NULL); \
 		igt_tokencat(__real_main, __LINE__)(); \
 		igt_exit(); \
 	} \
 	static void igt_tokencat(__real_main, __LINE__)(void) \
 
 
+const char *igt_test_name(void);
 void igt_simple_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
 				struct option *extra_long_opts,
 				const char *help_str,
-				igt_opt_handler_t extra_opt_handler);
+				igt_opt_handler_t extra_opt_handler,
+				void *handler_data);
 
 /**
  * igt_simple_init:
@@ -219,19 +233,21 @@ void igt_simple_init_parse_opts(int *argc, char **argv,
  * #igt_simple_main block instead of stitching the test's main() function together
  * manually.
  */
-#define igt_simple_init(argc, argv) igt_simple_init_parse_opts(&argc, argv, NULL, NULL, NULL, NULL);
+#define igt_simple_init(argc, argv) \
+	igt_simple_init_parse_opts(&argc, argv, NULL, NULL, NULL, NULL, NULL);
 
 /**
  * igt_simple_main:
  *
  * This is a magic control flow block used instead of a main() function for
  * simple tests. Open-coding the main() function is only recommended if
- * the test needs to parse additional cmdline arguments of its own.
+ * the test needs to parse additional command line arguments of its own.
  */
 #define igt_simple_main \
 	static void igt_tokencat(__real_main, __LINE__)(void); \
 	int main(int argc, char **argv) { \
-		igt_simple_init_parse_opts(&argc, argv, NULL, NULL, NULL, NULL); \
+		igt_simple_init_parse_opts(&argc, argv, NULL, NULL, NULL, \
+					   NULL, NULL); \
 		igt_tokencat(__real_main, __LINE__)(); \
 		igt_exit(); \
 	} \
@@ -248,8 +264,8 @@ void __igt_skip_check(const char *file, const int line,
 void igt_success(void);
 
 void igt_fail(int exitcode) __attribute__((noreturn));
-__attribute__((format(printf, 7, 8)))
-void __igt_fail_assert(int exitcode, const char *domain, const char *file,
+__attribute__((format(printf, 6, 7)))
+void __igt_fail_assert(const char *domain, const char *file,
 		       const int line, const char *func, const char *assertion,
 		       const char *format, ...)
 	__attribute__((noreturn));
@@ -265,7 +281,7 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_assert(expr) \
 	do { if (!(expr)) \
-		__igt_fail_assert(99, IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, #expr , NULL); \
+		__igt_fail_assert(IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, #expr , NULL); \
 	} while (0)
 
 /**
@@ -282,7 +298,7 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_assert_f(expr, f...) \
 	do { if (!(expr)) \
-		__igt_fail_assert(99, IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, #expr , f); \
+		__igt_fail_assert(IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, #expr , f); \
 	} while (0)
 
 /**
@@ -327,20 +343,65 @@ void igt_exit(void) __attribute__((noreturn));
 	do { \
 		int __n1 = (n1), __n2 = (n2); \
 		if (__n1 cmp __n2) ; else \
-		__igt_fail_assert(99, IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
+		__igt_fail_assert(IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
 				  #n1 " " #cmp " " #n2, \
 				  "error: %d " #ncmp " %d\n", __n1, __n2); \
 	} while (0)
 
+/**
+ * igt_assert_cmpuint:
+ * @n1: first value
+ * @cmp: compare operator
+ * @ncmp: negated version of @cmp
+ * @n2: second value
+ *
+ * Like igt_assert_cmpint(), but for unsigned ints.
+ */
 #define igt_assert_cmpuint(n1, cmp, ncmp, n2) \
 	do { \
 		uint32_t __n1 = (n1), __n2 = (n2); \
 		if (__n1 cmp __n2) ; else \
-		__igt_fail_assert(99, IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
+		__igt_fail_assert(IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
 				  #n1 " " #cmp " " #n2, \
 				  "error: %#x " #ncmp " %#x\n", __n1, __n2); \
 	} while (0)
 
+/**
+ * igt_assert_cmpu64:
+ * @n1: first value
+ * @cmp: compare operator
+ * @ncmp: negated version of @cmp
+ * @n2: second value
+ *
+ * Like igt_assert_cmpuint(), but for larger ints.
+ */
+#define igt_assert_cmpu64(n1, cmp, ncmp, n2) \
+	do { \
+		uint64_t __n1 = (n1), __n2 = (n2); \
+		if (__n1 cmp __n2) ; else \
+		__igt_fail_assert(IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
+				  #n1 " " #cmp " " #n2, \
+				  "error: %#llx " #ncmp " %#llx\n", (long long)__n1, (long long)__n2); \
+	} while (0)
+
+/**
+ * igt_assert_cmpdouble:
+ * @n1: first value
+ * @cmp: compare operator
+ * @ncmp: negated version of @cmp
+ * @n2: second value
+ *
+ * Like igt_assert_cmpint(), but for doubles.
+ */
+#define igt_assert_cmpdouble(n1, cmp, ncmp, n2) \
+	do { \
+		double __n1 = (n1), __n2 = (n2); \
+		if (__n1 cmp __n2) ; else \
+		__igt_fail_assert(IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
+				  #n1 " " #cmp " " #n2, \
+				  "error: %#lf " #ncmp " %#lf\n", __n1, __n2); \
+	} while (0)
+
 /**
  * igt_assert_eq:
  * @n1: first integer
@@ -353,8 +414,34 @@ void igt_exit(void) __attribute__((noreturn));
  * of simply printing the stringified expression.
  */
 #define igt_assert_eq(n1, n2) igt_assert_cmpint(n1, ==, !=, n2)
+
+/**
+ * igt_assert_eq_u32:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Like igt_assert_eq(), but for uint32_t.
+ */
 #define igt_assert_eq_u32(n1, n2) igt_assert_cmpuint(n1, ==, !=, n2)
 
+/**
+ * igt_assert_eq_u64:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Like igt_assert_eq_u32(), but for uint64_t.
+ */
+#define igt_assert_eq_u64(n1, n2) igt_assert_cmpu64(n1, ==, !=, n2)
+
+/**
+ * igt_assert_eq_double:
+ * @n1: first double
+ * @n2: second double
+ *
+ * Like igt_assert_eq(), but for doubles.
+ */
+#define igt_assert_eq_double(n1, n2) igt_assert_cmpdouble(n1, ==, !=, n2)
+
 /**
  * igt_assert_neq:
  * @n1: first integer
@@ -368,12 +455,39 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_assert_neq(n1, n2) igt_assert_cmpint(n1, !=, ==, n2)
 
+/**
+ * igt_assert_neq_u32:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Like igt_assert_neq(), but for uint32_t.
+ */
+#define igt_assert_neq_u32(n1, n2) igt_assert_cmpuint(n1, !=, ==, n2)
+
+/**
+ * igt_assert_neq_u64:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Like igt_assert_neq_u32(), but for uint64_t.
+ */
+#define igt_assert_neq_u64(n1, n2) igt_assert_cmpu64(n1, !=, ==, n2)
+
+/**
+ * igt_assert_neq_double:
+ * @n1: first double
+ * @n2: second double
+ *
+ * Like igt_assert_neq(), but for doubles.
+ */
+#define igt_assert_neq_double(n1, n2) igt_assert_cmpdouble(n1, !=, ==, n2)
+
 /**
  * igt_assert_lte:
  * @n1: first integer
  * @n2: second integer
  *
- * Fails (sub-)test if the second integers is greater than the first.
+ * Fails (sub-)test if the second integer is strictly smaller than the first.
  * Beware that for now this only works on integers.
  *
  * Like igt_assert(), but displays the values being compared on failure instead
@@ -386,7 +500,7 @@ void igt_exit(void) __attribute__((noreturn));
  * @n1: first integer
  * @n2: second integer
  *
- * Fails (sub-)test if the second integers is strictly smaller than the first.
+ * Fails (sub-)test if the second integer is smaller than or equal to the first.
  * Beware that for now this only works on integers.
  *
  * Like igt_assert(), but displays the values being compared on failure instead
@@ -394,6 +508,18 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_assert_lt(n1, n2) igt_assert_cmpint(n1, <, >=, n2)
 
+/**
+ * igt_assert_fd:
+ * @fd: file descriptor
+ *
+ * Fails (sub-) test if the given file descriptor is invalid.
+ *
+ * Like igt_assert(), but displays the values being compared on failure instead
+ * of simply printing the stringified expression.
+ */
+#define igt_assert_fd(fd) \
+	igt_assert_f(fd >= 0, "file descriptor " #fd " failed\n");
+
 /**
  * igt_require:
  * @expr: condition to test
@@ -406,7 +532,7 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_require(expr) do { \
 	if (!(expr)) igt_skip_check(#expr , NULL); \
-	else igt_debug("Test requirement passed: "#expr"\n"); \
+	else igt_debug("Test requirement passed: %s\n", #expr); \
 } while (0)
 
 /**
@@ -421,7 +547,7 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_skip_on(expr) do { \
 	if ((expr)) igt_skip_check("!(" #expr ")" , NULL); \
-	else igt_debug("Test requirement passed: !("#expr")\n"); \
+	else igt_debug("Test requirement passed: !(%s)\n", #expr); \
 } while (0)
 
 /**
@@ -440,7 +566,7 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_require_f(expr, f...) do { \
 	if (!(expr)) igt_skip_check(#expr , f); \
-	else igt_debug("Test requirement passed: "#expr"\n"); \
+	else igt_debug("Test requirement passed: %s\n", #expr); \
 } while (0)
 
 /**
@@ -459,7 +585,7 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_skip_on_f(expr, f...) do { \
 	if ((expr)) igt_skip_check("!("#expr")", f); \
-	else igt_debug("Test requirement passed: !("#expr")\n"); \
+	else igt_debug("Test requirement passed: !(%s)\n", #expr); \
 } while (0)
 
 /* fork support code */
@@ -647,6 +773,21 @@ extern enum igt_log_level igt_log_level;
 	} while (0)
 
 
-void igt_set_timeout(unsigned int seconds);
+void igt_set_timeout(unsigned int seconds,
+		     const char *op);
+
+void igt_reset_timeout(void);
+
+FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
+		       const char* filename);
+/**
+ * igt_fopen_data:
+ * @filename: filename to open.
+ *
+ * Open a datafile for test, first try from installation directory
+ * then from build directory.
+ */
+#define igt_fopen_data(filename) \
+	__igt_fopen_data(IGT_SRCDIR, IGT_DATADIR, filename)
 
 #endif /* IGT_CORE_H */
