@@ -35,10 +35,12 @@
 #include <xf86drmMode.h>
 
 #include "igt_fb.h"
+#include "ioctl_wrappers.h"
 
 /* Low-level helpers with kmstest_ prefix */
 
 enum pipe {
+        PIPE_ANY = -1,
         PIPE_A = 0,
         PIPE_B,
         PIPE_C,
@@ -70,7 +72,7 @@ enum port {
  * kmstest_port_name:
  * @port: display plane
  *
- * Returns: String represnting @port, e.g. "A".
+ * Returns: String representing @port, e.g. "A".
  */
 #define kmstest_port_name(port) ((port) + 'A')
 
@@ -127,26 +129,6 @@ enum kmstest_force_connector_state {
 	FORCE_CONNECTOR_OFF
 };
 
-
-/**
- * kmstest_generic_edid:
- * @EDID_XGA: 1024x768
- * @EDID_SXGA: 1280x1024
- * @EDID_UXGA: 1600x1200
- * @EDID_WSXGA: 1680x1050
- * @EDID_FHD: 1920x1080
- * @MAX_EDIDS: Size of #generic_edid array
- */
-enum kmstest_generic_edid {
-	EDID_XGA,   /* 1024x768 */
-	EDID_SXGA,  /* 1280x1024 */
-	EDID_UXGA,  /* 1600x1200 */
-	EDID_WSXGA, /* 1680x1050 */
-	EDID_FHD,   /* 1920x1080 */
-
-	MAX_EDIDS
-};
-
 bool kmstest_force_connector(int fd, drmModeConnector *connector,
 			     enum kmstest_force_connector_state state);
 void kmstest_edid_add_3d(const unsigned char *edid, size_t length, unsigned char *new_edid_ptr[], size_t *new_length);
@@ -201,6 +183,7 @@ typedef struct {
 	unsigned int position_changed : 1;
 	unsigned int panning_changed  : 1;
 	unsigned int rotation_changed : 1;
+	unsigned int size_changed     : 1;
 	/*
 	 * drm_plane can be NULL for primary and cursor planes (when not
 	 * using the atomic modeset API)
@@ -212,6 +195,8 @@ typedef struct {
 
 	/* position within pipe_src_w x pipe_src_h */
 	int crtc_x, crtc_y;
+	/* size within pipe_src_w x pipe_src_h */
+	int crtc_w, crtc_h;
 	/* panning offset within the fb */
 	unsigned int pan_x, pan_y;
 	igt_rotation_t rotation;
@@ -224,6 +209,9 @@ struct igt_pipe {
 #define IGT_MAX_PLANES	4
 	int n_planes;
 	igt_plane_t planes[IGT_MAX_PLANES];
+	uint64_t background; /* Background color MSB BGR 16bpc LSB */
+	uint32_t background_changed : 1;
+	uint32_t background_property;
 };
 
 typedef struct {
@@ -234,6 +222,8 @@ typedef struct {
 	char *name;
 	bool valid;
 	unsigned long pending_crtc_idx_mask;
+	bool use_override_mode;
+	drmModeModeInfo override_mode;
 } igt_output_t;
 
 struct igt_display {
@@ -256,6 +246,7 @@ int  igt_display_get_n_pipes(igt_display_t *display);
 
 const char *igt_output_name(igt_output_t *output);
 drmModeModeInfo *igt_output_get_mode(igt_output_t *output);
+void igt_output_override_mode(igt_output_t *output, drmModeModeInfo *mode);
 void igt_output_set_pipe(igt_output_t *output, enum pipe pipe);
 igt_plane_t *igt_output_get_plane(igt_output_t *output, enum igt_plane plane);
 
@@ -266,8 +257,14 @@ static inline bool igt_plane_supports_rotation(igt_plane_t *plane)
 
 void igt_plane_set_fb(igt_plane_t *plane, struct igt_fb *fb);
 void igt_plane_set_position(igt_plane_t *plane, int x, int y);
+void igt_plane_set_size(igt_plane_t *plane, int w, int h);
 void igt_plane_set_panning(igt_plane_t *plane, int x, int y);
 void igt_plane_set_rotation(igt_plane_t *plane, igt_rotation_t rotation);
+void igt_crtc_set_background(igt_pipe_t *pipe, uint64_t background);
+void igt_fb_set_position(struct igt_fb *fb, igt_plane_t *plane,
+	uint32_t x, uint32_t y);
+void igt_fb_set_size(struct igt_fb *fb, igt_plane_t *plane,
+	uint32_t w, uint32_t h);
 
 void igt_wait_for_vblank(int drm_fd, enum pipe pipe);
 
@@ -278,16 +275,18 @@ void igt_wait_for_vblank(int drm_fd, enum pipe pipe);
 #define for_each_pipe(display, pipe)					\
 	for (pipe = 0; pipe < igt_display_get_n_pipes(display); pipe++)	\
 
-/*
- * Can be used with igt_output_set_pipe() to mean we don't care about the pipe
- * that should drive this output
- */
-#define PIPE_ANY	(-1)
+#define for_each_plane_on_pipe(display, pipe, plane)			\
+	for (int i__ = 0; (plane) = &(display)->pipes[(pipe)].planes[i__], \
+		     i__ < (display)->pipes[(pipe)].n_planes; i__++)
 
 #define IGT_FIXED(i,f)	((i) << 16 | (f))
 
 void igt_enable_connectors(void);
 void igt_reset_connectors(void);
 
+#define EDID_LENGTH 128
+const unsigned char* igt_kms_get_base_edid(void);
+
+
 #endif /* __IGT_KMS_H__ */
 
