@@ -25,6 +25,7 @@
  *
  */
 
+#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -36,10 +37,8 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
+#include <time.h>
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_io.h"
 
 #define LOCAL_I915_EXEC_NO_RELOC (1<<11)
 #define LOCAL_I915_EXEC_HANDLE_LUT (1<<12)
@@ -100,22 +99,11 @@ static int sysfs_write(const char *name, int value)
 	return 0;
 }
 
-static int dcmp(const void *A, const void *B)
+static uint64_t elapsed(const struct timespec *start,
+		const struct timespec *end,
+		int loop)
 {
-	double a = *(double *)A, b = *(double *)B;
-	if (a < b)
-		return -1;
-	else if (a > b)
-		return 1;
-	else
-		return 0;
-}
-
-static double elapsed(const struct timeval *start,
-		      const struct timeval *end,
-		      int loop)
-{
-	return (1e6*(end->tv_sec - start->tv_sec) + (end->tv_usec - start->tv_usec))/loop;
+	return (1000000000ULL*(end->tv_sec - start->tv_sec) + (end->tv_nsec - start->tv_nsec))/loop;
 }
 
 static void loop(int fd, uint32_t handle, unsigned ring_id, const char *ring_name)
@@ -145,30 +133,29 @@ static void loop(int fd, uint32_t handle, unsigned ring_id, const char *ring_nam
 	gem_sync(fd, handle);
 
 	for (count = 1; count <= SLOW_QUICK(1<<17, 1<<4); count <<= 1) {
-		const int reps = 9;
-		double t[reps], sum;
+		const int reps = 7;
+		igt_stats_t stats;
 		int n;
 
+		igt_stats_init_with_size(&stats, reps);
+
 		for (n = 0; n < reps; n++) {
-			struct timeval start, end;
+			struct timespec start, end;
 			int loops = count;
-			gettimeofday(&start, NULL);
+			usleep(200000); /* wait 200ms for the hw to go back to sleep */
+			clock_gettime(CLOCK_MONOTONIC, &start);
 			while (loops--)
 				do_ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &execbuf);
 			gem_sync(fd, handle);
-			gettimeofday(&end, NULL);
-			t[n] = elapsed(&start, &end, count);
+			clock_gettime(CLOCK_MONOTONIC, &end);
+			igt_stats_push(&stats, elapsed(&start, &end, count));
 		}
 
-		qsort(t, reps, sizeof(double), dcmp);
-		sum = 0;
-		for (n = 2; n < reps-2; n++)
-			sum += t[n];
-		sum /= reps - 4;
-
 		igt_info("Time to exec x %d:		%7.3fµs (ring=%s)\n",
-				count, sum, ring_name);
+			 count, igt_stats_get_trimean(&stats)/1000, ring_name);
 		fflush(stdout);
+
+		igt_stats_fini(&stats);
 	}
 }
 
@@ -189,6 +176,8 @@ static void set_min_freq(void)
 	int min = sysfs_read("gt_RPn_freq_mhz");
 	igt_require(min > 0);
 	igt_debug("Setting min/max to %dMHz\n", min);
+	(void)sysfs_write("gt_idle_freq_mhz", min);
+	(void)sysfs_write("gt_boost_freq_mhz", min);
 	igt_require(sysfs_write("gt_min_freq_mhz", min) == 0 &&
 		    sysfs_write("gt_max_freq_mhz", min) == 0);
 }
@@ -198,6 +187,8 @@ static void set_max_freq(void)
 	int max = sysfs_read("gt_RP0_freq_mhz");
 	igt_require(max > 0);
 	igt_debug("Setting min/max to %dMHz\n", max);
+	(void)sysfs_write("gt_idle_freq_mhz", max);
+	(void)sysfs_write("gt_boost_freq_mhz", max);
 	igt_require(sysfs_write("gt_max_freq_mhz", max) == 0 &&
 		    sysfs_write("gt_min_freq_mhz", max) == 0);
 }
@@ -213,14 +204,16 @@ igt_main
 		{ "-max", set_max_freq },
 		{ NULL, NULL },
 	}, *r;
-	int min = -1, max = -1;
+	int min = -1, max = -1, boost = -1, idle = -1;
 	uint32_t handle = 0;
 
 	igt_fixture {
-		device = drm_open_any();
+		device = drm_open_driver(DRIVER_INTEL);
 
 		min = sysfs_read("gt_min_freq_mhz");
 		max = sysfs_read("gt_max_freq_mhz");
+		boost = sysfs_read("gt_boost_freq_mhz");
+		idle = sysfs_read("gt_idle_freq_mhz");
 
 		handle = gem_create(device, 4096);
 		gem_write(device, handle, 0, batch, sizeof(batch));
@@ -249,6 +242,10 @@ igt_main
 			sysfs_write("gt_min_freq_mhz", min);
 		if (max > 0)
 			sysfs_write("gt_max_freq_mhz", max);
+		if (boost > 0)
+			sysfs_write("gt_boost_freq_mhz", boost);
+		if (idle > 0)
+			sysfs_write("gt_idle_freq_mhz", idle);
 		close(device);
 	}
 }
