@@ -26,6 +26,7 @@
  */
 
 #define _GNU_SOURCE
+#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -37,9 +38,6 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "igt_debugfs.h"
 
 struct local_i915_gem_mmap_v2 {
 	uint32_t handle;
@@ -65,7 +63,6 @@ mmap_bo(int fd, uint32_t handle)
 	void *ptr;
 
 	ptr = gem_mmap__wc(fd, handle, 0, OBJECT_SIZE, PROT_READ | PROT_WRITE);
-	igt_assert(ptr && ptr != MAP_FAILED);
 
 	return ptr;
 }
@@ -104,7 +101,7 @@ test_invalid_flags(int fd)
 	gp.value = &val;
 
 	/* Do we have the new mmap_ioctl? */
-	do_ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp);
+	drmIoctl(fd, DRM_IOCTL_I915_GETPARAM, &gp);
 
 	if (val >= 1) {
 		/*
@@ -186,10 +183,8 @@ test_read_write2(int fd, enum test_read_write order)
 	set_domain(fd, handle);
 
 	r = gem_mmap__wc(fd, handle, 0, OBJECT_SIZE, PROT_READ);
-	igt_assert(r != MAP_FAILED);
 
 	w = gem_mmap__wc(fd, handle, 0, OBJECT_SIZE, PROT_READ | PROT_WRITE);
-	igt_assert(w != MAP_FAILED);
 
 	if (order == READ_BEFORE_WRITE) {
 		val = *(uint32_t *)r;
@@ -291,10 +286,8 @@ test_write_cpu_read_wc(int fd, int force_domain)
 	handle = gem_create(fd, OBJECT_SIZE);
 
 	dst = gem_mmap__wc(fd, handle, 0, OBJECT_SIZE, PROT_READ);
-	igt_assert(dst != (uint32_t *)MAP_FAILED);
 
 	src = gem_mmap__cpu(fd, handle, 0, OBJECT_SIZE, PROT_WRITE);
-	igt_assert(src != (uint32_t *)MAP_FAILED);
 
 	memset(src, 0xaa, OBJECT_SIZE);
 	if (force_domain)
@@ -318,10 +311,8 @@ test_write_gtt_read_wc(int fd)
 	set_domain(fd, handle);
 
 	dst = gem_mmap__wc(fd, handle, 0, OBJECT_SIZE, PROT_READ);
-	igt_assert(dst != (uint32_t *)MAP_FAILED);
 
 	src = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_WRITE);
-	igt_assert(src != (uint32_t *)MAP_FAILED);
 
 	memset(src, 0xaa, OBJECT_SIZE);
 	igt_assert(memcmp(dst, src, OBJECT_SIZE) == 0);
@@ -443,7 +434,7 @@ igt_main
 		OBJECT_SIZE = 1 * 1024 * 1024;
 
 	igt_fixture
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 
 	igt_subtest("invalid-flags")
 		test_invalid_flags(fd);
