@@ -22,9 +22,7 @@
  *
  */
 
-#include "igt_core.h"
-#include "igt_kms.h"
-#include "drmtest.h"
+#include "igt.h"
 
 IGT_TEST_DESCRIPTION("Tests 3D mode setting.");
 
@@ -37,7 +35,7 @@ igt_simple_main
 	size_t length;
 	int mode_count, connector_id;
 
-	drm_fd = drm_open_any_master();
+	drm_fd = drm_open_driver_master(DRIVER_INTEL);
 	res = drmModeGetResources(drm_fd);
 
 	igt_assert(drmSetClientCap(drm_fd, DRM_CLIENT_CAP_STEREO_3D, 1) >= 0);
@@ -74,7 +72,7 @@ igt_simple_main
 			mode_count++;
 	}
 
-	igt_assert(mode_count == 15);
+	igt_assert_eq(mode_count, 13);
 
 	/* set 3D modes */
 	igt_info("Testing:\n");
@@ -102,7 +100,7 @@ igt_simple_main
 
 		/* create stereo framebuffer */
 		fb_id = igt_create_stereo_fb(drm_fd, &connector->modes[i],
-					     igt_bpp_depth_to_drm_format(32, 32),
+					     igt_bpp_depth_to_drm_format(32, 24),
 					     LOCAL_DRM_FORMAT_MOD_NONE);
 
 		ret = drmModeSetCrtc(drm_fd, config.crtc->crtc_id, fb_id, 0, 0,
@@ -117,6 +115,4 @@ igt_simple_main
 
 	drmModeFreeConnector(connector);
 	free(edid);
-
-	igt_exit();
 }
