@@ -129,6 +129,28 @@ struct event_state {
 	int seq_step;
 };
 
+static void dump_event_state(const struct event_state *es)
+{
+	igt_debug("name = %s\n"
+		  "last_ts = %lu.%lu usec\n"
+		  "last_received_ts = %lu.%lu usec\n"
+		  "last_seq = %u\n"
+		  "current_ts = %lu.%lu usec\n"
+		  "current_received_ts = %lu.%lu usec\n"
+		  "current_seq = %u\n"
+		  "count = %u\n"
+		  "seq_step = %d\n",
+		  es->name,
+		  es->last_ts.tv_sec, es->last_ts.tv_usec,
+		  es->last_received_ts.tv_sec, es->last_received_ts.tv_usec,
+		  es->last_seq,
+		  es->current_ts.tv_sec, es->current_ts.tv_usec,
+		  es->current_received_ts.tv_sec, es->current_received_ts.tv_usec,
+		  es->current_seq,
+		  es->count,
+		  es->seq_step);
+}
+
 struct test_output {
 	int mode_valid;
 	drmModeModeInfo kmode[4];
@@ -136,6 +158,7 @@ struct test_output {
 	drmModeConnector *kconnector[4];
 	uint32_t _connector[4];
 	uint32_t _crtc[4];
+	int _pipe[4];
 	int count; /* 1:1 mapping between crtc:connector */
 	int flags;
 	int pipe; /* primary pipe for vblank */
@@ -547,7 +570,7 @@ static void page_flip_handler(int fd, unsigned int frame, unsigned int sec,
 
 static double frame_time(struct test_output *o)
 {
-	return 1000.0 * 1000.0 / o->kmode[0].vrefresh;
+	return 1000.0 * o->kmode[0].htotal * o->kmode[0].vtotal / o->kmode[0].clock;
 }
 
 static void *vblank_wait_thread_func(void *data)
@@ -616,6 +639,8 @@ static void check_state(struct test_output *o, struct event_state *es)
 	struct timeval diff;
 	double usec_interflip;
 
+	dump_event_state(es);
+
 	timersub(&es->current_ts, &es->current_received_ts, &diff);
 	if (!analog_tv_connector(o)) {
 		igt_assert_f(diff.tv_sec < 0 || (diff.tv_sec == 0 && diff.tv_usec <= 2000),
@@ -828,7 +853,7 @@ static int set_mode(struct test_output *o, uint32_t fb, int x, int y)
 {
 	int n;
 
-	for (n = 0; n < o->count; n++) {
+	for (n = o->count - 1; n >= 0; n--) {
 		if (fb == 0) {
 			int ret = drmModeSetCrtc(drm_fd, o->_crtc[n],
 						 0, 0, 0,
@@ -1078,6 +1103,7 @@ static void connector_find_preferred_mode(uint32_t connector_id, int crtc_idx,
 	o->kconnector[0] = config.connector;
 	o->kencoder[0] = config.encoder;
 	o->_crtc[0] = config.crtc->crtc_id;
+	o->_pipe[0] = config.pipe;
 	o->kmode[0] = config.default_mode;
 	o->mode_valid = 1;
 
@@ -1146,11 +1172,13 @@ found:
 	o->kconnector[0] = config[0].connector;
 	o->kencoder[0] = config[0].encoder;
 	o->_crtc[0] = config[0].crtc->crtc_id;
+	o->_pipe[0] = config[0].pipe;
 	o->kmode[0] = *mode[0];
 
 	o->kconnector[1] = config[1].connector;
 	o->kencoder[1] = config[1].encoder;
 	o->_crtc[1] = config[1].crtc->crtc_id;
+	o->_pipe[1] = config[1].pipe;
 	o->kmode[1] = *mode[1];
 
 	drmModeFreeCrtc(config[0].crtc);
@@ -1209,7 +1237,7 @@ static void check_final_state(struct test_output *o, struct event_state *es,
 		int count = es->count;
 
 		count *= o->seq_step;
-		expected = elapsed * o->kmode[0].vrefresh / (1000 * 1000);
+		expected = elapsed / frame_time(o);
 		igt_assert_f(count >= expected * 99/100 && count <= expected * 101/100,
 			     "dropped frames, expected %d, counted %d, encoder type %d\n",
 			     expected, count, o->kencoder[0]->encoder_type);
@@ -1326,22 +1354,32 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	switch (crtc_count) {
 	case 1:
 		connector_find_preferred_mode(o->_connector[0], crtc_idxs[0], o);
+		if (!o->mode_valid)
+			return;
 		snprintf(test_name, sizeof(test_name),
-			  "%s on crtc %d, connector %d",
-			  igt_subtest_name(), o->_crtc[0], o->_connector[0]);
+			 "%s on pipe %s, connector %s-%d",
+			 igt_subtest_name(),
+			 kmstest_pipe_name(o->_pipe[0]),
+			 kmstest_connector_type_str(o->kconnector[0]->connector_type),
+			 o->kconnector[0]->connector_type_id);
 		break;
 	case 2:
 		connector_find_compatible_mode(crtc_idxs[0], crtc_idxs[1], o);
+		if (!o->mode_valid)
+			return;
 		snprintf(test_name, sizeof(test_name),
-			 "%s on crtc %d:%d, connector %d:%d",
-			 igt_subtest_name(), o->_crtc[0], o->_crtc[1],
-			 o->_connector[0], o->_connector[1]);
+			 "%s on pipe %s:%s, connector %s-%d:%s-%d",
+			 igt_subtest_name(),
+			 kmstest_pipe_name(o->_pipe[0]),
+			 kmstest_pipe_name(o->_pipe[1]),
+			 kmstest_connector_type_str(o->kconnector[0]->connector_type),
+			 o->kconnector[0]->connector_type_id,
+			 kmstest_connector_type_str(o->kconnector[1]->connector_type),
+			 o->kconnector[1]->connector_type_id);
 		break;
 	default:
 		igt_assert(0);
 	}
-	if (!o->mode_valid)
-		return;
 
 	igt_assert_eq(o->count, crtc_count);
 
@@ -1366,7 +1404,7 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 					 tiling, &o->fb_info[0]);
 	o->fb_ids[1] = igt_create_fb_with_bo_size(drm_fd, o->fb_width, o->fb_height,
 					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
-					 tiling, &o->fb_info[1], bo_size);
+					 tiling, &o->fb_info[1], bo_size, 0);
 	o->fb_ids[2] = igt_create_fb(drm_fd, o->fb_width, o->fb_height,
 					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
 					 LOCAL_I915_FORMAT_MOD_X_TILED, &o->fb_info[2]);
@@ -1387,12 +1425,15 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	for (i = 0; i < o->count; i++)
 		kmstest_dump_mode(&o->kmode[i]);
 
+	kmstest_unset_all_crtcs(drm_fd, resources);
+
 	if (set_mode(o, o->fb_ids[0], 0, 0)) {
 		/* We may fail to apply the mode if there are hidden
 		 * constraints, such as bandwidth on the third pipe.
 		 */
 		igt_assert_f(crtc_count > 1 || crtc_idxs[0] < 2,
 			     "set_mode may only fail on the 3rd pipe or in multiple crtc tests\n");
+		igt_info("\n%s: SKIPPED\n\n", test_name);
 		goto out;
 	}
 	igt_assert(fb_is_bound(o, o->fb_ids[0]));
